import { world, system, BlockPermutation, ItemStack, GameMode } from "@minecraft/server";
import { decreaseItemStack, spawnItem } from "../../ntk_functions.js";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:glow_bird_egg_block_on_player_interact", {
    onPlayerInteract: eventData => {
      const player = eventData.player;
      const block = eventData.block;
      const blockLocation = block.location;
      const playerEquippable = player.getComponent("equippable");
      const itemUsed = playerEquippable.getEquipment("Mainhand");
      const blockState = block.permutation.getState("minecraft:cardinal_direction");
      const blockAllState = block.permutation.getAllStates();
      
      if (itemUsed === undefined && !player.isSneaking || itemUsed.typeId !== "nicothekid:glow_bird_egg" && !player.isSneaking) {
        if (block.typeId === "nicothekid:glow_bird_egg_1_block") {
          spawnItem(block, "nicothekid:glow_bird_egg", 1);
          block.dimension.playSound("block.itemframe.remove_item", blockLocation, { pitch: 0.8, volume: 1.0 });
          block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          return;
        }
        if (block.typeId === "nicothekid:glow_bird_egg_2_block") {
          spawnItem(block, "nicothekid:glow_bird_egg", 1)
          block.dimension.playSound("block.itemframe.remove_item", blockLocation, { pitch: 0.8, volume: 1.0 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:glow_bird_egg_1_block", blockAllState ));
          return;
        }
        if (block.typeId === "nicothekid:glow_bird_egg_3_block") {
          spawnItem(block, "nicothekid:glow_bird_egg", 1)
          block.dimension.playSound("block.itemframe.remove_item", blockLocation, { pitch: 0.8, volume: 1.0 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:glow_bird_egg_2_block", blockAllState));
          return;
        }
        if (block.typeId === "nicothekid:glow_bird_egg_4_block") {
          spawnItem(block, "nicothekid:glow_bird_egg", 1)
          block.dimension.playSound("block.itemframe.remove_item", blockLocation, { pitch: 0.8, volume: 1.0 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:glow_bird_egg_3_block", blockAllState));
          return;
        }
      }
      else if (itemUsed.typeId === "nicothekid:glow_bird_egg") {
        if (block.typeId === "nicothekid:glow_bird_egg_1_block") {
          decreaseItemStack(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("block.itemframe.add_item", blockLocation, { pitch: 1.0, volume: 0.5 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:glow_bird_egg_2_block", blockAllState));
          return;
        }
        if (block.typeId === "nicothekid:glow_bird_egg_2_block") {
          decreaseItemStack(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("block.itemframe.add_item", blockLocation, { pitch: 1.0, volume: 0.5 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:glow_bird_egg_3_block", blockAllState));
          return;
        }
        if (block.typeId === "nicothekid:glow_bird_egg_3_block") {
          decreaseItemStack(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("block.itemframe.add_item", blockLocation, { pitch: 1.0, volume: 0.5 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:glow_bird_egg_4_block", blockAllState));
          return;
        }
      }
      else return;
    }
  });
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:glow_bird_egg_block_on_tick", {
    onTick: eventData => {
      const block = eventData.block;
      const blockLocation = block.location;
      if (block.permutation.matches("nicothekid:glow_bird_egg_1_block")) {
        system.run(() => {
          block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          block.dimension.playSound("block.turtle_egg.crack", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          return;
        });
      }
      if (block.permutation.matches("nicothekid:glow_bird_egg_2_block")) {
        system.run(() => {
          block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          block.dimension.playSound("block.turtle_egg.crack", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          return;
        });
      }
      if (block.permutation.matches("nicothekid:glow_bird_egg_3_block")) {
        system.run(() => {
          block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          block.dimension.playSound("block.turtle_egg.crack", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          return;
        });
      }
      if (block.permutation.matches("nicothekid:glow_bird_egg_4_block")) {
        system.run(() => {
          block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          block.dimension.playSound("block.turtle_egg.crack", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          block.dimension.runCommandAsync(`summon nicothekid:glow_bird ${blockLocation.x + 0.5} ${blockLocation.y} ${blockLocation.z + 0.5} ~ ~ event:spawn_from_egg`);
          return;
        });
      }
    }
  });
});