import { world, system, BlockPermutation, GameMode } from "@minecraft/server";
import { decreaseItemStack } from "../../ntk_functions.js";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:moss_berry_bush_on_player_interact", {
    onPlayerInteract: eventData => {
      const player = eventData.player;
      const block = eventData.block;
      const blockLocation = block.location;
      const playerEquippable = player.getComponent("equippable");
      const itemUsed = playerEquippable.getEquipment("Mainhand");
      
      if (itemUsed === undefined) return;
      if (block.typeId === "nicothekid:moss_berry_bush" && itemUsed.typeId === "minecraft:bone_meal") {
        if (block.permutation.matches("nicothekid:moss_berry_bush", { "nicothekid:growth": 0 })) {
          decreaseItemStack(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:moss_berry_bush", { "nicothekid:growth": 1 }));
          return;
        }
        if (block.permutation.matches("nicothekid:moss_berry_bush", { "nicothekid:growth": 1 })) {
          decreaseItemStack(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:moss_berry_bush", { "nicothekid:growth": 2 }));
          return;
        }
      }
      else return;
    }
  });
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:moss_berry_bush_on_random_tick", {
    onRandomTick: eventData => {
      const block = eventData.block;
      const blockLocation = block.location;
      const blockBelow = block.below(1);
      if (block.typeId === "nicothekid:moss_berry_bush") {
        let randomChance = Math.random();
        if (block.permutation.matches("nicothekid:moss_berry_bush", { "nicothekid:growth": 0 })) {
          if (blockBelow.typeId === "minecraft:farmland") {
            if (randomChance <= 0.75) {
              block.setPermutation(BlockPermutation.resolve("nicothekid:moss_berry_bush", { "nicothekid:growth": 1 }));
              return;
            }
            else return;
          }
          else {
            if (randomChance <= 0.1) {
              block.setPermutation(BlockPermutation.resolve("nicothekid:moss_berry_bush", { "nicothekid:growth": 1 }));
              return;
            }
            else return;
          }
        }
        if (block.permutation.matches("nicothekid:moss_berry_bush", { "nicothekid:growth": 1 })) {
          if (blockBelow.typeId === "minecraft:farmland") {
            if (randomChance <= 0.75) {
              block.setPermutation(BlockPermutation.resolve("nicothekid:moss_berry_bush", { "nicothekid:growth": 2 }));
              return;
            }
            else return;
          }
          else {
            if (randomChance <= 0.1) {
              block.setPermutation(BlockPermutation.resolve("nicothekid:moss_berry_bush", { "nicothekid:growth": 2 }));
              return;
            }
            else return;
          }
        }
      }
      else return;
    }
  });
})
