import { world, system, BlockPermutation, ItemStack, GameMode } from "@minecraft/server";
import { decreaseItemStack, spawnItem } from "../../ntk_functions.js";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:short_mycelium_sprouts_on_player_destroy", {
    onPlayerDestroy: eventData => {
      const player = eventData.player;
      const destroyedBlock = eventData.destroyedBlockPermutation;
      const block = eventData.block;
      const blockLocation = block.location;
      const playerEquippable = player.getComponent("equippable");
      const itemUsed = playerEquippable.getEquipment("Mainhand");
      
      if (player.matches({ gameMode: GameMode.creative })) return;
      if (destroyedBlock.type.id === "nicothekid:short_mycelium_sprouts") {
        if (itemUsed === undefined) {
          return;
        }
        else {
          if (itemUsed.typeId === "minecraft:shears") {
            spawnItem(block, "nicothekid:short_mycelium_sprouts", 1);
            return;
          }
          else return;
        }
      }
      else return;
    }
  });
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:short_mycelium_sprouts_on_player_interact", {
    onPlayerInteract: eventData => {
      const player = eventData.player;
      const block = eventData.block;
      const blockLocation = block.location;
      const playerEquippable = player.getComponent("equippable");
      const itemUsed = playerEquippable.getEquipment("Mainhand");
      
      if (itemUsed === undefined) return;
      if (block.typeId === "nicothekid:short_mycelium_sprouts") {
        if (itemUsed.typeId === "minecraft:bone_meal") {
          decreaseItemStack(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:tall_mycelium_sprouts"));
          return;
        }
        if (itemUsed.typeId === "nicothekid:glowcelium_bone_meal") {
          decreaseItemStack(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
          block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
          block.setPermutation(BlockPermutation.resolve("nicothekid:tall_glowcelium_sprouts"));
          return;
        }
      }
      else return;
    }
  });
})