import { world, system, BlockPermutation, ItemStack, GameMode } from "@minecraft/server";
import { decreaseItemStack, spawnItem } from "../../ntk_functions.js";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:tall_mycelium_sprouts_on_player_destroy", {
    onPlayerDestroy: eventData => {
      const player = eventData.player;
      const destroyedBlock = eventData.destroyedBlockPermutation;
      const block = eventData.block;
      const blockLocation = block.location;
      const playerEquippable = player.getComponent("equippable");
      const itemUsed = playerEquippable.getEquipment("Mainhand");
      let randomChance = Math.random() * 100.0;
      
      if (player.matches({ gameMode: GameMode.creative })) return;
      if (destroyedBlock.type.id === "nicothekid:tall_mycelium_sprouts") {
        if (itemUsed === undefined) {
          if (randomChance <= 5.0) {
            spawnItem(block, "nicothekid:short_mycelium_sprouts", 1);
            return;
          }
          else return;
        }
        else {
          if (itemUsed.typeId === "minecraft:shears") {
            spawnItem(block, "nicothekid:tall_mycelium_sprouts", 1);
            return;
          }
          else {
            const itemEnchantable = itemUsed.getComponent("enchantable");
            if (itemEnchantable) {
              const silkTouchEnchanted = itemEnchantable.getEnchantment("silk_touch");
              if (silkTouchEnchanted) return;
              const fortuneEnchanted = itemUsed.getComponent("enchantable").getEnchantment("fortune");
              if (itemUsed.getComponent("enchantable").hasEnchantment("fortune")) {
                if (fortuneEnchanted.level == 1) {
                  if (randomChance <= 8.0) {
                    spawnItem(block, "nicothekid:short_mycelium_sprouts", 1);
                    return;
                  }
                  else return;
                }
                if (fortuneEnchanted.level == 2) {
                  if (randomChance <= 10.0) {
                    spawnItem(block, "nicothekid:short_mycelium_sprouts", 1);
                    return;
                  }
                  else return;
                }
                if (fortuneEnchanted.level == 3) {
                  if (randomChance <= 15.0) {
                    spawnItem(block, "nicothekid:short_mycelium_sprouts", 1);
                    return;
                  }
                  else return;
                }
              }
              else {
                if (randomChance <= 10.0) {
                  spawnItem(block, "nicothekid:short_mycelium_sprouts", 1);
                  return;
                }
                else return;
              }
            }
            else {
              if (randomChance <= 10.0) {
                spawnItem(block, "nicothekid:short_mycelium_sprouts", 1);
                return;
              }
              else return;
            }
          }
        }
      }
      else return;
    }
  });
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:tall_mycelium_sprouts_on_player_interact", {
    onPlayerInteract: eventData => {
      const player = eventData.player;
      const block = eventData.block;
      const blockLocation = block.location;
      const playerEquippable = player.getComponent("equippable");
      const itemUsed = playerEquippable.getEquipment("Mainhand");
      
      if (itemUsed === undefined) return;
      if (block.typeId === "nicothekid:tall_mycelium_sprouts" && itemUsed.typeId === "nicothekid:glowcelium_bone_meal") {
        decreaseItemStack(player, itemUsed, 1, "Mainhand");
        block.dimension.playSound("item.bone_meal.use", blockLocation, { pitch: 1.0, volume: 1.0 });
        block.dimension.spawnParticle("minecraft:crop_growth_emitter", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
        block.setPermutation(BlockPermutation.resolve("nicothekid:tall_glowcelium_sprouts"));
        return;
      }
      else return;
    }
  });
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:tall_mycelium_sprouts_on_random_tick", {
    onRandomTick: eventData => {
      const block = eventData.block;
      const blockLocation = block.location;
      if (block.typeId === "nicothekid:tall_mycelium_sprouts") {
        if (block.permutation.matches("nicothekid:tall_mycelium_sprouts", { "nicothekid:sprouts_particles": false })) {
          block.setPermutation(BlockPermutation.resolve("nicothekid:tall_mycelium_sprouts", { "nicothekid:sprouts_particles": true }))
          return;
        }
        else return
      }
      else return;
    }
  });
  initEvent.blockComponentRegistry.registerCustomComponent("nicothekid:tall_mycelium_sprouts_on_tick", {
    onTick: eventData => {
      const block = eventData.block;
      const blockLocation = block.location;
      if (block.typeId === "nicothekid:tall_mycelium_sprouts") {
        if (block.permutation.matches("nicothekid:tall_mycelium_sprouts", { "nicothekid:sprouts_particles": true })) {
          block.dimension.spawnParticle("nicothekid:mycelium_sprouts_particle", { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 })
          return;
        }
      }
      else return;
    }
  });
})
