import { world, system, BlockPermutation, GameMode } from "@minecraft/server";
import { instantDiggableBlocks } from "../ntk_listing.js";
import { damageItemDurability, damageItemDurabilityFixer } from "../ntk_functions.js";

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  initEvent.itemComponentRegistry.registerCustomComponent("nicothekid:custom_swords_nce_on_hit_entity", {
    onHitEntity: eventData => {
      const damager = eventData.attackingEntity;
      const target = eventData.hitEntity;
      const itemUsed = eventData.itemStack;
      if (target === undefined || itemUsed === undefined) return;
      const { x, y, z } = target.location;
      const damagerEquippable = damager.getComponent("equippable");
      
      if (itemUsed.hasTag("minecraft:is_sword")) {
        if (target.matches({ excludeFamilies: [ "spirit" ] })) {
          // Durabilty Damage:
          damageItemDurabilityFixer(damager, itemUsed, 1, "Mainhand");
        };
      }
      else return;
    }
  });
  initEvent.itemComponentRegistry.registerCustomComponent("nicothekid:custom_swords_nce_on_mine_block", {
    onMineBlock: eventData => {
      const player = eventData.source;
      const itemUsed = eventData.itemStack;
      const block = eventData.block;
      const blockPermutation = eventData.minedBlockPermutation;
      const playerEquippable = player.getComponent("equippable");
      
      if (itemUsed.hasTag("minecraft:is_sword")) {
        if (!instantDiggableBlocks.includes(blockPermutation.type.id) && !blockPermutation.hasTag("nicothekid:instant_diggable")) {
          damageItemDurability(player, itemUsed, 2, "Mainhand");
        }
        else return;
      }
      else return;
    }
  });
});
