import { world, system, BlockPermutation, ItemStack, GameMode } from "@minecraft/server";
import { damageItemDurability } from "../ntk_functions.js";
import { vanillaWoods } from "../ntk_listing.js";

world.beforeEvents.itemUseOn.subscribe(eventData => {
  const player = eventData.source;
  const itemUsed = eventData.itemStack;
  const block = eventData.block;
  const blockLocation = block.location;
  const blockAllState = block.permutation.getAllStates();
  const playerEquippable = player.getComponent("equippable");
  const customAxes = [
    "nicothekid:brass_axe",
    "nicothekid:bronze_axe",
    "nicothekid:olivine_axe",
    "nicothekid:tin_axe",
    "nicothekid:zinc_axe"
  ]
  const customHoes = [
    "nicothekid:brass_hoe",
    "nicothekid:bronze_hoe",
    "nicothekid:olivine_hoe",
    "nicothekid:tin_hoe",
    "nicothekid:zinc_hoe"
  ]
  const customShovels = [
    "nicothekid:brass_shovel",
    "nicothekid:bronze_shovel",
    "nicothekid:olivine_shovel",
    "nicothekid:tin_shovel",
    "nicothekid:zinc_shovel"
  ]
  
  if (itemUsed === undefined) return;
  if (customAxes.includes(itemUsed.typeId) || customHoes.includes(itemUsed.typeId) || customShovels.includes(itemUsed.typeId)) {
    if (customAxes.includes(itemUsed.typeId)) {
      if (vanillaWoods.includes(block.typeId)) {
        system.run(() => {
          damageItemDurability(player, itemUsed, 1, "Mainhand");
          if (block.permutation.matches("minecraft:oak_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:oak_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:spruce_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:spruce_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:birch_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:birch_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:jungle_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:jungle_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:acacia_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:acacia_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:dark_oak_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:dark_oak_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:mangrove_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:mangrove_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:cherry_log")) {
            block.dimension.playSound("step.cherry_wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:cherry_wood")) {
            block.dimension.playSound("step.cherry_wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:crimson_stem")) {
            block.dimension.playSound("use.stem", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:crimson_hyphae")) {
            block.dimension.playSound("use.stem", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:warped_stem")) {
            block.dimension.playSound("use.stem", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:warped_hyphae")) {
            block.dimension.playSound("use.stem", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:bamboo_block")) {
            block.dimension.playSound("step.bamboo_wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:pale_oak_log")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
          if (block.permutation.matches("minecraft:pale_oak_wood")) {
            block.dimension.playSound("use.wood", blockLocation, { pitch: 0.8, volume: 1.0 });
          }
        });
      }
    }
    if (customHoes.includes(itemUsed.typeId)) {
      const vanillaFarmSoil = [
        "minecraft:coarse_dirt",
        "minecraft:dirt",
        "minecraft:dirt_with_roots",
        "minecraft:grass_block",
        "minecraft:grass_path"
      ];
      if (vanillaFarmSoil.includes(block.typeId)) {
        system.run(() => {
          const blockAbove = block.dimension.getBlock(blockLocation).above(1);
          if (!blockAbove.isAir) return;
          damageItemDurability(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("use.gravel", blockLocation, { pitch: 0.8, volume: 1.0 });
        });
      }
    }
    if (customShovels.includes(itemUsed.typeId)) {
      const vanillaCampfire = [
        "minecraft:campfire",
        "minecraft:soul_campfire"
      ];
      const vanillaPathSoil = [
        "minecraft:coarse_dirt",
        "minecraft:dirt",
        "minecraft:dirt_with_roots",
        "minecraft:grass_block",
        "minecraft:mycelium",
        "minecraft:podzol"
      ];
      if (vanillaCampfire.includes(block.typeId)) {
        system.run(() => {
          const blockState = block.permutation.getState("minecraft:cardinal_direction");
          const pitchRange = Math.random() * (2.4 - 1.8) + 1.8;
          if (block.permutation.matches("minecraft:campfire", { "extinguished": false, "minecraft:cardinal_direction": blockState })) {
            damageItemDurability(player, itemUsed, 1, "Mainhand");
            block.dimension.playSound("random.fizz", blockLocation, { pitch: pitchRange, volume: 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:campfire", { "extinguished": true, "minecraft:cardinal_direction": blockState }));
          }
          if (block.permutation.matches("minecraft:soul_campfire", { "extinguished": false, "minecraft:cardinal_direction": blockState })) {
            damageItemDurability(player, itemUsed, 1, "Mainhand");
            block.dimension.playSound("random.fizz", blockLocation, { pitch: pitchRange, volume: 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:soul_campfire", { "extinguished": true, "minecraft:cardinal_direction": blockState }));
          }
        });
      }
      if (vanillaPathSoil.includes(block.typeId)) {
        system.run(() => {
          const blockAbove = block.dimension.getBlock(blockLocation).above(1);
          if (!blockAbove.isAir) return;
          damageItemDurability(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("use.grass", blockLocation, { pitch: 0.8, volume: 1.0 });
          if (block.permutation.matches("minecraft:dirt", { dirt_type: "normal" })) {
            block.setPermutation(BlockPermutation.resolve("minecraft:grass_path"));
          }
          if (block.permutation.matches("minecraft:dirt", { dirt_type: "coarse" })) {
            block.setPermutation(BlockPermutation.resolve("minecraft:grass_path"));
          }
          if (block.permutation.matches("minecraft:dirt_with_roots")) {
            block.setPermutation(BlockPermutation.resolve("minecraft:grass_path"));
          }
          /*if (block.permutation.matches("minecraft:grass_block")) {
            block.setPermutation(BlockPermutation.resolve("minecraft:grass_path"));
          }*/
          if (block.permutation.matches("minecraft:mycelium")) {
            block.setPermutation(BlockPermutation.resolve("minecraft:grass_path"));
          }
          if (block.permutation.matches("minecraft:podzol")) {
            block.setPermutation(BlockPermutation.resolve("minecraft:grass_path"));
          }
        });
      }
      if (block.typeId === "minecraft:snow_layer") {
        system.run(() => {
          const pitchRange = Math.random() * (1.0 - 0.8) + 0.8;
          damageItemDurability(player, itemUsed, 1, "Mainhand");
          block.dimension.playSound("dig.snow", blockLocation, { pitch: pitchRange, volume: 1.0 });
          if (block.permutation.matches("minecraft:snow_layer", { height: 0 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 1);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
          if (block.permutation.matches("minecraft:snow_layer", { height: 1 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 1);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
          if (block.permutation.matches("minecraft:snow_layer", { height: 2 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 1);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
          if (block.permutation.matches("minecraft:snow_layer", { height: 3 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 2);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
          if (block.permutation.matches("minecraft:snow_layer", { height: 4 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 2);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
          if (block.permutation.matches("minecraft:snow_layer", { height: 5 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 3);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
          if (block.permutation.matches("minecraft:snow_layer", { height: 6 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 3);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
          if (block.permutation.matches("minecraft:snow_layer", { height: 7 })) {
            const spawnSnowball = new ItemStack("minecraft:snowball", 4);
            block.dimension.spawnItem(spawnSnowball, { x: blockLocation.x + 0.5, y: blockLocation.y + 0.5, z: blockLocation.z + 0.5 });
            block.setPermutation(BlockPermutation.resolve("minecraft:air"));
          }
        });
      }
    }
  }
  else return;
});